unit As400;

{ by Paul Toth (TothPaul@mygale.org) }
{ http://www.mygale.org/~tothpaul    }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, CaError, Cawin;

type
 {
  The Custom400 component has an Error property, so each API can be called by
   Error:=API(Parms);
  The SetError call OnError Event or Call the virtual InternalError procedure
 }
  TCustom400=class(TComponent)
  private
   fError:integer;
   EOnError:TNotifyEvent;
   Procedure SetError(Value:Integer);
  Protected
   Procedure InternalError; virtual; abstract;
  Public
   Property Error:integer read fError write SetError;
  Published
   Property OnError:TNotifyEvent read EOnError write EOnError;
  end;

 { Some type }

  TSysName=String[10]; TSysNameZ=Array[0..10] of char;
  TUserId=String[10];  TUserIdZ=Array[0..10] of char;

 { AS400 provide a link to an AS/400 }

  TAS400 = class(TCustom400)
  private
    { Private-dclarations }
    fHandle:Hwnd;
    fActive:boolean;
    fSysNameZ:TSysNameZ; { can be *DEFAULT }
    fSysUsed:TSysNameZ;  { when SysNameZ=*DEFAULT, SysUsed use GetDefaultSysName API }
    fUserIdZ:TUserIdZ;   { user id in current system }
    Procedure SetActive(Value:boolean);
    Function  GetSysName:TSysName;
    Procedure SetSysName(Value:TSysName);
  protected
    { Protected-dclarations }
    Procedure Loaded; override;
  public
    { Public-dclarations }
    constructor Create(AOwner:TComponent); override;
    Procedure InternalError; override;
    Function  UserId:TUserId;
    Function  SysUsed:TSysName;
    Function  SysNameZ:PChar;
    Function  Connect:boolean;
  published
    { Published-dclarations }
    Property  Active:boolean read fActive write SetActive;
    Property  SysName:TSysName read GetSysName write SetSysName;
  end;

 {
  Child400 is a component with a Host property
  the ValidHost function raise an exception when Host=nil
 }

  TChild400 = class (TCustom400)
  Private
   fHost:TAs400;
   Procedure SetHost(Value:TAs400);
  Protected
   Procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   Procedure CloseConnexion; virtual;
   Function ValidHost:TAs400;
  Published
   Property Host:TAs400 read fHost write SetHost;
  end;

{ SysList fill the parm list with available systems }

 Procedure SysList(List:TStrings);

implementation

Const
 MaxSystems=32;
Type
 TSystem=Array[0..9] of char;
 TSystems=Array[0..MaxSystems] of TSystem;

Procedure SysList(List:TStrings);
 Var
  Count,i:integer;
  Systems:^TSystems;
  S:TSysNameZ;
 begin
  List.Clear;
  if Not EHNAPPC_IsRouterLoaded(0) then exit;
  New(Systems);
  if EHNAPPC_QuerySystems(0,Count,Systems^)=0 then begin
   For i:=0 to Count-1 do begin
    StrLCopy(S,Systems^[i],10);
    List.add(StrPas(S));
   end;
  end;
  Dispose(Systems);
 end;

{ TCustom400 }

Procedure TCustom400.SetError(Value:Integer);
 begin
  fError:=Value;
  if (fError<>0) then begin
   if Assigned(EOnError) then
    EOnError(Self)
   else
    InternalError;
  end;
 end;

{ TAs400 }

Constructor TAs400.Create(AOwner:TComponent);
 begin
  inherited Create(AOwner);
  fHandle:=Application.Handle;
 end;

Procedure TAs400.Loaded;
 begin
  inherited Loaded;
  if fActive then Connect;
 end;

Procedure TAs400.InternalError;
 begin
  raise ECawin.CreateCaShow('RTR',Error);
 end;

Procedure TAs400.SetActive(Value:Boolean);
 begin
  if fActive<>Value then
   if (csLoading in ComponentState) then
    fActive:=Value
   else
    if Value then Connect else fActive:=False;
 end;

Function TAs400.Connect:boolean;
 begin
  if SysName='' then SysName:='*DEFAULT';
  if not EHNAPPC_IsRouterLoaded(fHandle) then raise ECawin.Create('Router not found');
  fActive:=(EHNAPPC_QueryUserId(fHandle,fSysUsed,fUserIDZ)=0);
 end;

Function TAs400.GetSysName:TSysName;
 begin
  Result:=StrPas(fSysNameZ);
 end;

Function TAs400.SysNameZ:PChar;
 begin
  Result:=fSysUsed;
 end;

Function TAs400.SysUsed:TSysName;
 begin
  Result:=StrPas(fSysUsed);
 end;

Procedure TAs400.SetSysName(Value:TSysName);
 begin
  if Value[1]='*' then
   Error:=EHNAPPC_GetDefaultSystem(fHandle,fSysUsed)
  else
   StrPCopy(fSysUsed,Value);
  StrPCopy(fSysNameZ,Value);
 end;

Function TAs400.UserId:TUserId;
 begin
  if fUserIdZ[0]=#0 then Result:='*NONE' else Result:=StrPas(fUserIdZ);
 end;

 { TChild400 }

Procedure TChild400.Notification(AComponent: TComponent; Operation: TOperation);
 begin
  inherited Notification(AComponent,Operation);
  if (AComponent=fHost) and (Operation=opRemove) then Host:=nil;
 end;

Procedure TChild400.SetHost(Value:TAs400);
 begin
  if fHost=Value then exit;
  if fHost<>nil then CloseConnexion;
  fHost:=Value;
 end;

Function TChild400.ValidHost:TAs400;
 begin
  if fHost=nil then
   raise ECawin.Create('Host property not set')
  else
   if fHost.Active then result:=fHost else raise ECawin.Create('Host not active');
 end;

Procedure TChild400.CloseConnexion;
 begin
  { this procedure have to be overrided }
 end;

end.
