unit Call400;

{ by Paul Toth (TothPaul@mygale.org) }
{ http://www.mygale.org/~tothpaul    }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, As400, Cawin;

type
  ECall400 = class(exception);
  TCall400 = class;

  TParmAccess=(paInput, paOutput, paBoth);

  TParm400=class(TPersistent)
   private
    fAccess:TParmAccess;
    fSize:word;
    fBind:PChar;
    Owner:TCall400;
    fName:string;
    procedure SetSize(Value:word);
   public
    Constructor Create(AOwner:TCall400);
    Destructor Destroy; override;
    Property Access:TParmAccess read fAccess write fAccess;
    Property Size:word read fSize write SetSize;
    Property Name:string read fName write fName;
    Procedure AddPgm(HWnd:HWND;PgmHandle:THandle);
  end;

  TCall400 = class(TChild400)
  private
    { Dclarations private }
    fHandle:THandle;
    fPgmName:string;
    fLibName:string;
    fParms:TList;
    function GetParm(Index:Integer):TParm400;
    function GetParmCount:integer;
  protected
    { Dclarations protected }
  public
    { Dclarations public }
    Constructor Create(AOwner:TComponent); override;
    Destructor Destroy; override;
    procedure InternalError; override;
    Property Parms[Index:integer]:TParm400 read GetParm;
    Property ParmCount:integer read GetParmCount;
    Function FindParm(Const ParmName:string):TParm400;
    Function Execute:boolean;
  published
    { Dclarations published }
    Property PgmName:string read fPgmName write fPgmName;
    Property LibName:string read fLibName write fLibName;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('CAWIN', [TCall400]);
end;

{ TCall400 }
Constructor TCall400.Create(AOwner:TComponent);
 begin
  inherited Create(AOwner);
  fHandle:=Application.Handle;
  fParms:=TList.Create;
 end;

Destructor TCall400.Destroy;
 begin
  While ParmCount>0 do Parms[0].Free;
  fParms.Free;
  inherited Destroy;
 end;

procedure TCall400.InternalError;
 begin
  raise ECall400.Create('Call/400 error $'+IntToHex(Error,4));
 end;

function TCall400.GetParm(Index:Integer):TParm400;
 begin
  Result:=fParms[Index];
 end;

function TCall400.GetParmCount:integer;
 begin
  Result:=fParms.Count;
 end;

function TCall400.FindParm(Const ParmName:string):TParm400;
 var
  i:integer;
 begin
  for i:=0 to ParmCount-1 do begin
   Result:=fParms[i];
   if Result.Name=ParmName then exit;
  end;
  Result:=nil;
 end;

Function TCall400.Execute:boolean;
 Var
  SysHandle:THandle;
  PgmHandle:THandle;
  i:integer;
 begin
  fPgmName[Length(fPgmName)+1]:=#0;
  fLibName[Length(fLibName)+1]:=#0;
  Error:=EHNDP_StartSys(fHandle,ValidHost.SysNameZ,'Delphi 16',SysHandle);
  Try
   Error:=EHNDP_CreatePgm(fHandle,@fPgmName[1],@fLibName[1],PgmHandle);
   Try
    For i:=0 to ParmCount-1 do Parms[i].AddPgm(fHandle,PgmHandle);
    Error:=EHNDP_CallPgm(fHandle,SysHandle,PgmHandle);
   Finally
    Error:=EHNDP_DeletePgm(fHandle,PgmHandle);
   end;
  Finally
   EHNDP_StopSys(fHandle,SysHandle);
  end;
 end;

{ TParm }
Constructor TParm400.Create(AOwner:TCall400);
 begin
  inherited Create;
  Owner:=AOwner;
  fSize:=0;
  Owner.fParms.Add(Self);
 end;

Destructor TParm400.Destroy;
 begin
  Size:=0;
  Owner.fParms.Remove(Self);
  inherited Destroy;
 end;

Procedure TParm400.SetSize(Value:word);
 begin
  if fSize>0 then FreeMem(fBind,fSize);
  fSize:=Value;
  if fSize>0 then GetMem(fBind,fSize);
 end;

Procedure TParm400.AddPgm(HWnd:HWND;PgmHandle:THandle);
 Const
  DataType:array[paInput..paBoth] of word = (0,1,2);
 begin
  EHNDP_AddParm(HWnd,PgmHandle,DataType[fAccess],fSize,fBind^);
 end;

end.
