unit Cmd400;

{ by Paul Toth (TothPaul@mygale.org) }
{ http://www.mygale.org/~tothpaul    }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, As400, CaError, Cawin, stdCtrls;

type
  ECmd400 = class(Exception);

  TCmd400 = class(TChild400)
  private
    { Private-dclarations }
    fOpen:boolean;
    fCommand:String;
    fLog:TMemo;
    fReply:PChar;
    fReplySize:Word;
    fReturnSize:word;
    fDoEvents:boolean;
    EWhileBusy:TNotifyEvent;
    EBeforeSubmit:TNotifyEvent;
    EAfterSubmit:TNotifyEvent;
    Procedure SetCommand(Value:string);
    Procedure SetReplySize(Value:word);
  protected
    { Protected-dclarations }
    Constructor Create(AOwner:TComponent); override;
    Destructor Destroy; override;
    Procedure Notification(AComponent:TComponent; Operation:TOperation); override;
    Procedure CloseConnexion; override;
    Procedure ReadError;
    Procedure CallBack;
  public
    { Public-dclarations }
    Procedure InternalError; override;
    Function Exec(Cmd:string):boolean;
  published
    { Published-dclarations }
    Property Command:string read fCommand write SetCommand stored false;
    Property Log:TMemo read fLog write fLog;
    Property ReplySize:word read fReplySize write SetReplySize;
    Property DoEvents:boolean read fDoEvents write fDoEvents;
    Property WhileBusy:TNotifyEvent read EWhileBusy write EWhileBusy;
    Property BeforeSubmit:TNotifyEvent read EBeforeSubmit write EBeforeSubmit;
    Property AfterSubmit:TNotifyEvent read EAfterSubmit write EAfterSubmit;
  end;

implementation

Const
 CallBackOwner:TCmd400=nil;

Function RemoteCommandCallBack:integer; export;
 begin
  if CallBackOwner<>nil then CallBackOwner.CallBack;
 end;

Constructor TCmd400.Create(AOwner:TComponent);
 begin
  Inherited Create(AOwner);
  fReplySize:=0;
  ReplySize:=255;
 end;

Destructor TCmd400.Destroy;
 begin
  CloseConnexion;
  ReplySize:=0;
  inherited Destroy;
 end;

procedure TCmd400.Notification(AComponent:TComponent; Operation:TOperation);
 begin
  inherited Notification(AComponent,Operation);
  if (Operation=opRemove)and(AComponent=fLog) then fLog:=nil;
 end;

Procedure TCmd400.CloseConnexion;
 begin
  if fOpen then EHNSR_StopConversation(0,ValidHost.SysNameZ);
 end;

Procedure TCmd400.InternalError;
 begin
  raise ECawin.CreateCAGet('SRW',Error);
 end;

Procedure TCmd400.CallBack;
 Var
  msg:TMsg;
 begin
  If Assigned(EWhileBusy) then
   EWhileBusy(Self)
  else
   if DoEvents then
    while PeekMessage(msg, 0, 0, 0, PM_REMOVE) do begin
     TranslateMessage(msg);
     DispatchMessage(msg);
    end;
 end;


Procedure TCmd400.SetCommand(Value:string);
 begin
  if fLog<>nil then fLog.Lines.Add('>'+Value);
  fCommand:=Value;
  Value:=Value+#0; { String to PChar }
  fReturnSize:=fReplySize;
  Try
   if Assigned(EBeforeSubmit) then EBeforeSubmit(Self);
   CallBackOwner:=Self;
   Error:=EHNSR_SubmitCommand(0,ValidHost.SysNameZ,@Value[1],fReply,fReturnSize,@RemoteCommandCallBack);
   fOpen:=True;
  Finally
   CallBackOwner:=nil;
   ReadError;
   if Assigned(EAfterSubmit) then EAfterSubmit(Self);
  end;
  fCommand:='';
 end;

Function TCmd400.Exec(Cmd:string):boolean;
 begin
  Result:=True;
  Try
   SetCommand(Cmd);
  except
   Result:=false;
  end;
 end;

Procedure TCmd400.ReadError;
 Var
  SevCode:word;
  MsgId:Array[0..7] of char;
  MSg:Array[0..255] of char;
 begin
  If fReturnSize>0 then begin
   While EHNSR_GetMessage(0,ValidHost.SysNameZ,SevCode,MsgId,Msg,256,fReply,fReturnSize)=0 do begin
    if StrPas(MsgId)<>'5814' then { no error }
     if fLog<>nil then fLog.Lines.Add(StrPas(MsgId)+' - '+StrPas(Msg));
   end;
  end;
 end;

Procedure TCmd400.SetReplySize(Value:word);
 begin
  if Value<>fReplySize then begin
   if fReplySize>0 then FreeMem(fReply,fReplySize);
   fReplySize:=Value;
   if fReplySize>0 then GetMem(fReply,fReplySize);
  end;
 end;

end.
