unit Tfr400;

{ by Paul Toth (TothPaul@mygale.org) }
{ http://www.mygale.org/~tothpaul    }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, As400, CaError, Cawin;

type
  ETfr400=class(Exception);

  TConversion=(cvNone, cvDOSRandom, cvDOSRandom2);

  TTfrStyle=(tsToPC, tsFromPC);

  TTfrStatus=set of (tsOpen,tsReading);

  TCustomTransfer=class(TChild400)
  Private
   tfrStatus:TTfrStatus;
   fTfrStyle:TTfrStyle;
   SysParm:Array[0..18] of Char;   { SYSTEM("SysName") }
   ConvId:LongInt;
   ConvCode:byte;
   Buffer:Array[0..4095] of char;
   RequestLen:word;
   Reply:Pchar;
   IcfSize,IcfCode:Integer;
   fConversion:TConversion;
   fTranslate:boolean;
   fSupportNullVar:boolean;
   fDoEvents:boolean;
   EBeforeRequest:TNotifyEvent;
   EAfterRequest:TNotifyEvent;
   EWhileBusy:TNotifyEvent;
   fEOF:boolean;
   Procedure SetConversion(Value:TConversion);
   Procedure SetTranslate(Value:boolean);
   Procedure SetSupportNullVar(Value:boolean);
  Protected
   Constructor Create(AOwner:TComponent); override;
   Destructor Destroy; override;
   Procedure CloseConnexion; override;
   Procedure InternalError; override;
   Procedure Transfer(Func:byte);
   Property Conversion:TConversion read fConversion write SetConversion;
   Property Translate:boolean read fTranslate write SetTranslate;
   Property SupportNullVar:boolean read fSupportNullVar write SetSupportNullVar;
   Procedure CallBack;
  Public
   Procedure CloseRequest;
   Procedure EndRequest;
   Property EOF:boolean read fEOF;
  Published
   Property BeforeRequest:TNotifyEvent read EBeforeRequest write EBeforeRequest;
   Property AfterRequest:TNotifyEvent read EAfterRequest write EAfterRequest;
   Property WhileBusy:TNotifyEvent read EWhileBusy write EWhileBusy;
   Property DoEvents:boolean read fDoEvents write fDoEvents;
   Property TfrStyle:TTfrStyle read fTfrStyle write fTfrStyle;
  end;

{ TTransfer }

  TLibName = string[10];
  TFileName = String[10];
  TMbrName=string[10];
  TTableName=string[21];

Const
 rqRequest     =01;
 rqGetTemplate =02;
 rqGetRecord   =03;
 rqCloseCurrent=04;
 rqCloseAll    =05;
 rqSetRecord   =06;
 rqEndRequest  =07;

 kLIB=$3E;
 kFIL=$4D;
 kMBR=$55;
 kFMT=$62;
 kFLD=$4C;

 LibD =60;
 FileD=75;
 MbrD =83;
 FmtD =96;
 FldD =74;

Type
  TFFD = record
   Case Kind:word of
    kLIB : (LibName:Array[0.. 9] of char;
            LibText:Array[0..49] of char);
    kFIL : (FileName:Array[0..20] of char;
            FileType:Array[0..3] of char;
            FileText:Array[0..49] of char);
    kMBR : (MbrName:Array[0..32] of char;
            MbrText:Array[0..49] of char);
    kFMT : (FmtName:Array[0..43] of char;
            FmtSize:word;
            FmtText:Array[0..49] of char);
    kFLD : (FldName:Array[0..9] of char;
            FldType:Array[0..4] of char;
            FldSize:word;
            FldNum :word;
            FldDec :word;
            FldText:Array[0..49] of char;
            FldNull:Array[0..2] of char);
   end;

  TAddRecord=Procedure(List:TStrings; Var FFD:TFFD) of object;

  TTfr400 = class(TCustomTransfer)
  private
    { Private-dclarations }
    fLibName:TLibName;
    fFileName:TFileName;
    fMbrName:TMbrName;
    fFileSize:Integer;
  protected
    { Protected-dclarations }
    Procedure AddLib(List:TStrings; Var FFD:TFFD);
    Procedure AddFile(List:TStrings; Var FFD:TFFD);
    Procedure AddMbr(List:TStrings; var FFD:TFFD);
    Procedure AddFmt(List:TStrings; var FFD:TFFD);
    Procedure AddFld(List:TStrings; var FFD:TFFD);
  public
    { Public-dclarations }
    Procedure BuildList(sql:string;List:TStrings;AddRecord:TAddRecord);
    Procedure LibList(List:TStrings);
    Procedure FileList(Lib:String; List:TStrings; Logical:boolean);
    Procedure MemberList(Table:string; List:TStrings);
    Procedure FormatList(Table:String; List:TStrings);
    Procedure FieldList(Table:String; List:TStrings);
    Function SqlRequest(Sql:string):boolean;
    Function Read(Var Data; DataSize:word):boolean;
    Function Write(Var Data; DataSize:word):boolean;
    Procedure OpenFile;
    Property FileSize:Integer read fFilesize;
  published
    { Published-dclarations }
    Property LibName:TLibName read fLibName write fLibName;
    Property FileName:TFileName read fFileName write fFileName;
    Property MbrName:TMbrName read fMbrName write fMbrName;
    Property Conversion;
    Property Translate;
    Property SupportNullVar;
  end;

Type
 TCallBack=function:integer;

Function RTrim(s:string):string;

Implementation

Function RTrim(s:string):string;
 begin
  Result:=s;
  While Result[Length(Result)]=' ' do dec(Result[0]);
 end;

Const
 CallBackOwner:TCustomTransfer=nil;

Function TransferCallBack:integer; export;
 begin
  if CallBackOwner<>nil then CallBackOwner.CallBack;
 end;

{ TCustomTransfer }

Constructor TCustomTransfer.Create(AOwner:TComponent);
 begin
  Inherited Create(AOwner);
  fTfrStyle:=tsToPC;
  TfrStatus:=[];
 end;

Destructor TCustomTransfer.Destroy;
 begin
  EndRequest;
  Inherited Destroy;
 end;

Procedure TCustomTransfer.CloseConnexion;
 begin
  EndRequest;
 end;

Procedure TCustomTransfer.CloseRequest;
 begin
  if tsReading in TfrStatus then begin
   Exclude(TfrStatus,tsReading);
   RequestLen:=0;
   Transfer(rqCloseCurrent);
  end;
 end;

Procedure TCustomTransfer.EndRequest;
 begin
  if tsOpen in TfrStatus then begin
   TfrStatus:=[];
   RequestLen:=0;
   Transfer(rqEndRequest);
  end;
 end;

Procedure TCustomTransfer.CallBack;
 Var
  msg:TMsg;
 begin
  If Assigned(EWhileBusy) then
   EWhileBusy(Self)
  else
   if DoEvents then
    while PeekMessage(msg, 0, 0, 0, PM_REMOVE) do begin
     TranslateMessage(msg);
     DispatchMessage(msg);
    end;
 end;

Procedure TCustomTransfer.SetConversion(Value:TConversion);
 begin
  fConversion:=Value;
  ConvCode:=Ord(Value);
  if ConvCode=2 then ConvCode:=$11; { cvRandom2 }
  if (fTranslate)and(ConvCode>0) then Inc(ConvCode);
  if fSupportNullVar then Inc(ConvCode,$80);
 end;

Procedure TCustomTransfer.SetTranslate(Value:boolean);
 begin
  if fTranslate<>Value then begin
   fTranslate:=Value;
   SetConversion(fConversion);
  end;
 end;

Procedure TCustomTransfer.SetSupportNullVar(Value:boolean);
 begin
  if fSupportNullVar<>Value then begin
   fSupportNullVar:=Value;
   SetConversion(fConversion);
  end;
 end;

Procedure TCustomTransfer.InternalError;
 begin
  if IcfSize>0 then
   Raise ECawin.Create(StrPas(Reply))
  else
   Raise ECawin.CreateCAGet('TFW',Error);
 end;

Procedure TCustomTransfer.Transfer(Func:byte);
 Var
  Err:integer;
 begin
  StrPCopy(SysParm,'SYSTEM('+ValidHost.SysUsed+')');
  fEOF:=False;
  Try
   If Assigned(EBeforeRequest) then EBeforeRequest(Self);
   CallBackOwner:=Self;
   Err:=EHNTF_STF(0,@TransferCallBack,ConvId,Func,Buffer,4096,RequestLen,
                  SysParm,ConvCode,Reply,IcfSize,IcfCode);
   fEOF:=(Err<>0);
   if fEOF then begin
    Exclude(TfrStatus,tsReading);
    if err<>$1FFF then Error:=err;
   end else
    if Func=rqRequest then Include(TfrStatus,tsReading);
  Finally
   CallBackOwner:=nil;
   If Assigned(EAfterRequest) then EAfterRequest(Self);
  end;
 end;

{ TTransfer }

Procedure TTfr400.AddLib(List:TStrings; Var FFD:TFFD);
 begin
  List.Add((RTrim(FFD.LibName)));
 end;

Procedure TTfr400.AddFile(List:TStrings; Var FFD:TFFD);
 begin
  List.Add((Copy(RTrim(FFD.FileName),Pos('/',FFD.FileName)+1,10)));
 end;

Procedure TTfr400.AddMbr(List:TStrings;var FFD:TFFD);
 var
  s:string;
 begin
  s:=Copy(RTrim(FFD.MbrName),Pos('(',FFD.MbrName)+1,11); dec(s[0]);
  List.Add((s));
 end;

Procedure TTfr400.AddFmt(List:TStrings; var FFD:TFFD);
 Var
  s:string;
 begin
  s:=Copy(RTrim(FFD.FmtName),Pos(',',FFD.FmtName)+1,11); dec(s[0]);
  List.add((s));
 end;

Procedure TTfr400.AddFld(List:TStrings; var FFD:TFFD);
 begin
  List.Add((RTrim(FFD.FldName)));
 end;

Procedure TTfr400.BuildList(sql:string;List:TStrings;AddRecord:TAddRecord);
 Var
  OldConv:word;
  OldCursor:TCursor;
  FFD:TFFD;
 begin
  OldConv:=ConvCode;
  ConvCode:=$81;
  OldCursor:=Screen.Cursor;
  Screen.Cursor:=crSQLWait;
  Try
   if SqlRequest(sql) then begin
    List.Clear;
    While Read(FFD,SizeOf(TFFD)) do AddRecord(List,FFD);
   end;
  Finally
   Screen.Cursor:=crDefault;
   ConvCode:=OldConv;
  end;
 end;

Procedure TTfr400.LibList(List:TStrings);
 begin
  BuildList('EXTRACT TABLES',List,AddLib);
 end;

Procedure TTfr400.FileList(Lib:string; List:TStrings; Logical:boolean);
 Const
  FType:array[false..true] of string[3]=('PHY','LFD');
 begin
  BuildList('EXTRACT TABLES ('+FType[Logical]+') '+Lib+'/',List,AddFile);
 end;

Procedure TTfr400.MemberList(Table:string; List:TStrings);
 begin
  BuildList('EXTRACT TABLES '+Table,List,AddMbr);
 end;

Procedure TTfr400.FormatList(Table:string; List:TStrings);
 begin
  BuildList('EXTRACT TABLES '+Table,List,AddFmt);
 end;

Procedure TTfr400.FieldList(Table:string; List:TStrings);
 begin
  BuildList('EXTRACT COLUMNS '+Table,List,AddFld);
 end;

Function TTfr400.SqlRequest(Sql:String):boolean;
 begin
  Result:=False;
  CloseRequest;
  RequestLen:=Length(Sql);
  Move(Sql[1],Buffer,RequestLen);
  Include(TfrStatus,tsOpen);
  Transfer(rqRequest);
  Result:=True;
 end;

Function TTfr400.Read(Var Data; DataSize:Word):boolean;
 begin
  Result:=False;
  RequestLen:=0;
  Transfer(rqGetRecord);
  Move(Reply[0],Data,DataSize);
  Result:=not fEOF;
 end;

Function TTfr400.Write(Var Data; DataSize:word):boolean;
 begin
  Result:=false;
  RequestLen:=DataSize;
  Move(Data,Buffer,DataSize);
  Transfer(rqSetRecord);
  Result:=true;
 end;

Procedure TTfr400.OpenFile;
 Var
  sql:string;
 begin
  if fTfrStyle=tsToPC then sql:='SELECT * FROM' else sql:='REPLACE * INTO';
  sql:=sql+' '+LibName+'/'+FileName;
  if MbrName<>'' then sql:=sql+'('+MbrName+')';
  SqlRequest(sql);
  fFileSize:=IcfSize;
 end;

end.
